@@ -36,6 +36,7 @@ panel.parallel <- function(z, subscripts,
                            lwd=superpose.line$lwd,
                            lty=superpose.line$lty, ...)
 {
+
     superpose.line <- trellis.par.get("superpose.line")
     reference.line <- trellis.par.get("reference.line")
 
@@ -48,15 +49,15 @@ panel.parallel <- function(z, subscripts,
     llim <- numeric(n.r)
     ulim <- numeric(n.r)
     dif <- numeric(n.r)
-    if(n.r>0)
+    if (n.r > 0)
         for(i in 1:n.r) {
             grid.lines(x = c(0,1), y = c(i,i),
                        default.units = "native",
                        gp = gpar(col = reference.line$col,
                        lwd = reference.line$lwd,
                        lty = reference.line$lty))
-            llim[i] <- range(z[,i])[1]
-            ulim[i] <- range(z[,i])[2]
+            llim[i] <- range(as.numeric(z[,i]))[1]
+            ulim[i] <- range(as.numeric(z[,i]))[2]
             dif[i] <- ulim[i] - llim[i]
         }
    
@@ -153,9 +154,9 @@ parallel <-
     foo$fontsize.small <- 8
 
     ## This is for cases like xlab/ylab = list(cex=2)
-    if (is.list(foo$xlab) && !is.character(foo$xlab$label))
-        foo$xlab$label <- "Scatter Plot Matrix"
-    if (is.list(foo$ylab) && !is.character(foo$ylab$label))
+    if (is.list(foo$xlab) && !is.characterOrExpression(foo$xlab$label))
+        foo$xlab$label <- "Parallel Coordinate Plot"
+    if (is.list(foo$ylab) && !is.characterOrExpression(foo$ylab$label))
         foo$ylab <- NULL
 
     ## Step 2: Compute scales.common (leaving out limits for now)
@@ -189,7 +190,7 @@ parallel <-
     have.ylim <- TRUE
     if (!is.null(foo$y.scales$limit)) {
         have.ylim <- TRUE
-        ylim <- foo$x.scales$limit
+        ylim <- foo$y.scales$limit
     }
     
     ## Step 4: Decide if log scales are being used:
@@ -287,7 +288,9 @@ parallel <-
                                panel.args.common = foo$panel.args.common,
                                panel.args = foo$panel.args,
                                aspect = aspect,
-                               nplots = nplots))
+                               nplots = nplots,
+                               x.axs = foo$x.scales$axs,
+                               y.axs = foo$y.scales$axs))
 
     class(foo) <- "trellis"
     foo
