@@ -17,14 +17,44 @@
 ### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 ### MA 02111-1307, USA
 
-.First.lib <- function(lib, pkg) {
+
+
+.onLoad <- function(lib, pkg) {
   library.dynam(pkg, pkg, lib )
-  if (!require(grid))
-      stop("lattice requires grid, but grid couldn't be loaded")
+  #import(grid) # will this do ?  apparently not
+  ## do we want grid functions visible
+  ##if (!require(grid))
+  ##    stop("lattice requires grid, but grid couldn't be loaded")
 }
 
+.LatticeEnv <- new.env()
+
 ## Need global variable to handle more in print.trellis
-.lattice.print.more <- FALSE
+assign(".lattice.print.more", FALSE, env = .LatticeEnv)
+assign("lattice.theme", list(), env = .LatticeEnv)
+
+.noGenerics <- TRUE
+
+.onUnload <- function(libpath)
+    library.dynam.unload("lattice", libpath)
+
+
+
+
+
+
+
+
+
+# old (pre NAMESPACE version)
+#.First.lib <- function(lib, pkg) {
+#  library.dynam(pkg, pkg, lib )
+#  if (!require(grid))
+#      stop("lattice requires grid, but grid couldn't be loaded")
+#}
+
+## Need global variable to handle more in print.trellis
+#.lattice.print.more <- FALSE
 
 
 
